(***********************************************************************



                    Mathematica-Compatible Notebook



This notebook can be used on any computer system with Mathematica 3.0,

MathReader 3.0, or any compatible application. The data for the notebook 

starts with the line of stars above.



To get the notebook into a Mathematica-compatible application, do one of 

the following:



* Save the data starting with the line of stars above into a file

  with a name ending in .nb, then open the file inside the application;



* Copy the data starting with the line of stars above to the

  clipboard, then use the Paste menu command inside the application.



Data for notebooks contains only printable 7-bit ASCII and can be

sent directly in email or through ftp in text mode.  Newlines can be

CR, LF or CRLF (Unix, Macintosh or MS-DOS style).



NOTE: If you modify the data for this notebook not in a Mathematica-

compatible application, you must delete the line below containing the 

word CacheID, otherwise Mathematica-compatible applications may try to 

use invalid cache data.



For more information on notebooks and Mathematica-compatible 

applications, contact Wolfram Research:

  web: http://www.wolfram.com

  email: info@wolfram.com

  phone: +1-217-398-0700 (U.S.)



Notebook reader applications are available free of charge from 

Wolfram Research.

***********************************************************************)



(*CacheID: 232*)





(*NotebookFileLineBreakTest

NotebookFileLineBreakTest*)

(*NotebookOptionsPosition[     59501,       1818]*)

(*NotebookOutlinePosition[     60329,       1847]*)

(*  CellTagsIndexPosition[     60285,       1843]*)

(*WindowFrame->Normal*)







Notebook[{

Cell[BoxData[

    StyleBox[\(\ Using\ Coloured\ Expressions\),

      "Title"]], "Input",

  TextAlignment->Center,

  TextJustification->0],



Cell["\<\

By  Dr. David Bailey, Salford Software

(db@salford-software.com)\

\>", "Text",

  TextAlignment->Center,

  TextJustification->0],



Cell[CellGroupData[{



Cell["Introduction", "Section"],



Cell[TextData[{

  "Despite the enormous range of sophisticated operations which are possible \

using ",

  StyleBox["Mathematica",

    FontSlant->"Italic"],

  ", manipulating expressions in arbitrary ways on a one-off basis can be \

remarkably  difficult. Beginners may just give up, but even more advanced \

users may find that the work required to devise a sequence of transformations \

is just not worth the effort, and in any case distracts from the work at \

hand. The ",

  StyleBox["Mathematica",

    FontSlant->"Italic"],

  " frontend provides some help, for example, it is only necessary to select \

(i.e. highlight) an expression and press a button corresponding to some \

desired operation to work on one part of an expression. However, this \

approach has several limitations.\n\nThe first problem is that selections are \

only visible temporarily and leave no record of what has been done in the \

notebook, they can also only be accessed by palettes of buttons. A more \

fundamental limitation is that it is only possible to select one part of an \

expression at once. Consider for example transforming this equation: "

}], "Text"],



Cell[BoxData[

    \(a\^2 + b\^2 + Log[a + b] + Log[c + d] - Log[a - b] - Log[c - d]\)], 

  "Input"],



Cell["Let us say that we wish to group the logarithms thus:", "Text"],



Cell[BoxData[

    \(\(\ a\^2 + b\^2 + Log[\(a + b\)\/\(a - b\)] + 

      Log[\(c + d\)\/\(c - d\)]\)\)], "Input"],



Cell[TextData[{

  "There is no easy way to perform this operation without writing a \

transformation rule that would be almost as complex as the equation to be \

transformed, nor is it easy to devise a palette button to perform this task, \

since all the terms are so similar. Of course, this is only a simple example, \

real world examples could be much more complicated than this. \n\nThis paper \

describes a technique of colouring expressions in a notebook to specify \

desired transformations. Several portions of an expression can be coloured \

with the same or different colours and the colours are retained in the \

notebook, thereby providing a record of work done. Once parts of an \

expression are coloured this provides a handle on which a variety of \

processes can operate.\n\nEarly sections of this paper will describe the use \

of this technique, later sections describe its implementation. Novice \

Mathematica users may wish to skip the implementation details. The full \

package is available at http://www.salford.co.uk/consultancy/mathematica, \

together with documentation and a copy of this paper in the form of a ",

  StyleBox["Mathematica",

    FontSlant->"Italic"],

  " notebook."

}], "Text"]

}, Open  ]],



Cell[CellGroupData[{



Cell["A simple example", "Section"],



Cell["Let us load the expression colouring package:", "Text"],



Cell[BoxData[

    \(<< \ "\<c:\maths\coloured_expressions.m\>"\)], "Input",

  CellLabel->"In[1]:="],



Cell["\<\

The package creates a palette which includes three coloured buttons. To use \

them let us consider the above expression. By selecting a part of this \

expression and clicking the red button we arrive at the following coloured \

expression:\

\>", "Text"],



Cell[BoxData[

    RowBox[{\(a\^2\), "+", \(b\^2\), "+", 

      TagBox[

        StyleBox[\(Log[a + b]\),

          FontColor->RGBColor[1, 0, 0]],

        Red], "+", \(Log[c + d]\), "-", \(Log[a - b]\), "-", 

      \(Log[c - d]\)}]], "Input"],



Cell["\<\

We can now select another part of the same expression and colour it red in \

the same way. Notice how, unlike raw selection, we have been able to pick out \

more than one part of the expression simultaneously:\

\>", "Text"],



Cell[BoxData[

    RowBox[{\(a\^2\), "+", \(b\^2\), "+", 

      TagBox[

        StyleBox[\(Log[a + b]\),

          FontColor->RGBColor[1, 0, 0]],

        Red], "+", \(Log[c + d]\), "-", 

      TagBox[

        StyleBox[\(Log[a - b]\),

          FontColor->RGBColor[1, 0, 0]],

        Red], "-", \(Log[c - d]\)}]], "Input"],



Cell["\<\

Now we can supply a simple rule that performs half of the desired \

transformation:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{\(a\^2\), "+", \(b\^2\), "+", 

        TagBox[

          StyleBox[\(Log[a + b]\),

            FontColor->RGBColor[1, 0, 0]],

          Red], "+", \(Log[c + d]\), "-", 

        TagBox[

          StyleBox[\(Log[a - b]\),

            FontColor->RGBColor[1, 0, 0]],

          Red], "-", \(Log[c - d]\)}], "/.", 

      \(Red[Log[x_]] - Red[Log[y_]] -> Log[x/y]\)}]], "Input",

  CellLabel->"In[6]:="],



Cell[BoxData[

    \(a\^2 + b\^2 + Log[\(a + b\)\/\(a - b\)] - Log[c - d] + Log[c + d]\)], 

  "Output",

  CellLabel->"Out[6]="]

}, Open  ]],



Cell["\<\

The other half of the transformation can be performed in exactly the same \

way, but we will use another colour, just for clarity:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

      \(a\^2\), "+", \(b\^2\), "+", \(Log[\(a + b\)\/\(a - b\)]\), "-", 

        TagBox[

          StyleBox[\(Log[c - d]\),

            FontColor->RGBColor[0, 1, 0]],

          Green], "+", 

        TagBox[

          StyleBox[\(Log[c + d]\),

            FontColor->RGBColor[0, 1, 0]],

          Green]}], "/.", \(Green[Log[x_]] - Green[Log[y_]] -> Log[x/y]\)}]], 

  "Input",

  CellLabel->"In[7]:="],



Cell[BoxData[

    \(a\^2 + b\^2 + Log[\(a + b\)\/\(a - b\)] + Log[\(c + d\)\/\(c - d\)]\)], 

  "Output",

  CellLabel->"Out[7]="]

}, Open  ]],



Cell["\<\

We could have combined these two operations together by applying both colours \

to the parts of the original expression and then applying both rules at once, \

but we will see how to perform the above transformation in a neater way still \

using two colours and some of the colour manipulation functions yet to be \

described.\

\>", "Text"],



Cell["\<\

To understand how the above example works it is useful to look at the \

FullForm of a coloured expression:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{"a", "+", 

        TagBox[

          StyleBox[\(b\ c\),

            FontColor->RGBColor[1, 0, 0]],

          Red]}], "//", "FullForm"}]], "Input",

  CellLabel->"In[8]:="],



Cell[BoxData[

    TagBox[

      StyleBox[\(Plus[a, Red[Times[b, c]]]\),

        ShowSpecialCharacters->False,

        ShowStringCharacters->True],

      FullForm]], "Output",

  CellLabel->"Out[8]//FullForm="]

}, Open  ]],



Cell["\<\

The colour is actually represented as a function of the same name. Note that \

this only applies to expressions coloured using the palette associated with \

the colouring package. If you colour text in some other way, e.g. by using \

the Mathematica menu, its FullForm representation will be unchanged. The \

colouring package has added rules to MakeBoxes to cause the three functions \

Red[], Green[], and Blue[] to be represented as coloured text tagged in a \

suitable way so that MakeExpression can back-convert coloured expressions \

into applications of these functions. Looking at this FullForm representation \

it is now obvious how the above transformation rule works. 



Because colour is represented as function applications, it is even possible \

to perform algebraic manipulations on expressions without losing the \

colour:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        RowBox[{"(", 

          RowBox[{"a", "+", 

            TagBox[

              StyleBox["b",

                FontColor->RGBColor[0, 0, 1]],

              Blue], "+", "c"}], ")"}], "^", "2"}], " ", "//", "Expand"}]], 

  "Input",

  CellLabel->"In[2]:="],



Cell[BoxData[

    RowBox[{\(a\^2\), "+", \(2\ a\ c\), "+", \(c\^2\), "+", 

      RowBox[{"2", " ", "a", " ", 

        TagBox[

          StyleBox["b",

            FontColor->RGBColor[0, 0, 1]],

          Blue]}], "+", 

      RowBox[{"2", " ", "c", " ", 

        TagBox[

          StyleBox["b",

            FontColor->RGBColor[0, 0, 1]],

          Blue]}], "+", 

      SuperscriptBox[

        TagBox[

          StyleBox["b",

            FontColor->RGBColor[0, 0, 1]],

          Blue], "2"]}]], "Output",

  CellLabel->"Out[2]="]

}, Open  ]],



Cell[TextData[{

  "Also, because ",

  StyleBox["Mathematica",

    FontSlant->"Italic"],

  " does not 'know' how to evaluate the colour functions, they can be used to \

produce output which is otherwise hard to obtain because of unwanted \

evaluations, for example:"

}], "Text"],



Cell[BoxData[

    \(Fact[1] := 1; \nFact[\(-1\)] := \(-1\); 

    Fact[s_] := 

      Times@@\((FactorInteger[s] /. {a_Integer, b_Integer} -> Red[a]^b)\); 

    \)], "Input",

  CellLabel->"In[98]:="],



Cell[CellGroupData[{



Cell[BoxData[

    \(Series[BesselJ[0, z], {z, 0, 16}] /. 

      Rational[a_, b_] :> Fact[a]/Fact[b]\)], "Input",

  CellLabel->"In[99]:="],



Cell[BoxData[

    InterpretationBox[

      RowBox[{"1", "-", 

        FractionBox[\(z\^2\), 

          SuperscriptBox[

            TagBox[

              StyleBox["2",

                FontColor->RGBColor[1, 0, 0]],

              Red], "2"]], "+", 

        FractionBox[\(z\^4\), 

          SuperscriptBox[

            TagBox[

              StyleBox["2",

                FontColor->RGBColor[1, 0, 0]],

              Red], "6"]], "-", 

        FractionBox[\(z\^6\), 

          RowBox[{

            SuperscriptBox[

              TagBox[

                StyleBox["2",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "8"], " ", 

            SuperscriptBox[

              TagBox[

                StyleBox["3",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "2"]}]], "+", 

        FractionBox[\(z\^8\), 

          RowBox[{

            SuperscriptBox[

              TagBox[

                StyleBox["2",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "14"], " ", 

            SuperscriptBox[

              TagBox[

                StyleBox["3",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "2"]}]], "-", 

        FractionBox[\(z\^10\), 

          RowBox[{

            SuperscriptBox[

              TagBox[

                StyleBox["2",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "16"], " ", 

            SuperscriptBox[

              TagBox[

                StyleBox["3",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "2"], " ", 

            SuperscriptBox[

              TagBox[

                StyleBox["5",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "2"]}]], "+", 

        FractionBox[\(z\^12\), 

          RowBox[{

            SuperscriptBox[

              TagBox[

                StyleBox["2",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "20"], " ", 

            SuperscriptBox[

              TagBox[

                StyleBox["3",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "4"], " ", 

            SuperscriptBox[

              TagBox[

                StyleBox["5",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "2"]}]], "-", 

        FractionBox[\(z\^14\), 

          RowBox[{

            SuperscriptBox[

              TagBox[

                StyleBox["2",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "22"], " ", 

            SuperscriptBox[

              TagBox[

                StyleBox["3",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "4"], " ", 

            SuperscriptBox[

              TagBox[

                StyleBox["5",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "2"], " ", 

            SuperscriptBox[

              TagBox[

                StyleBox["7",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "2"]}]], "+", 

        FractionBox[\(z\^16\), 

          RowBox[{

            SuperscriptBox[

              TagBox[

                StyleBox["2",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "30"], " ", 

            SuperscriptBox[

              TagBox[

                StyleBox["3",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "4"], " ", 

            SuperscriptBox[

              TagBox[

                StyleBox["5",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "2"], " ", 

            SuperscriptBox[

              TagBox[

                StyleBox["7",

                  FontColor->RGBColor[1, 0, 0]],

                Red], "2"]}]], "+", 

        InterpretationBox[\(O[z]\^18\),

          SeriesData[ z, 0, {}, 0, 18, 1]]}],

      SeriesData[ z, 0, {1, 0, 

        Times[ -1, 

          Power[ 

            Red[ 2], -2]], 0, 

        Power[ 

          Red[ 2], -6], 0, 

        Times[ -1, 

          Power[ 

            Red[ 2], -8], 

          Power[ 

            Red[ 3], -2]], 0, 

        Times[ 

          Power[ 

            Red[ 2], -14], 

          Power[ 

            Red[ 3], -2]], 0, 

        Times[ -1, 

          Power[ 

            Red[ 2], -16], 

          Power[ 

            Red[ 3], -2], 

          Power[ 

            Red[ 5], -2]], 0, 

        Times[ 

          Power[ 

            Red[ 2], -20], 

          Power[ 

            Red[ 3], -4], 

          Power[ 

            Red[ 5], -2]], 0, 

        Times[ -1, 

          Power[ 

            Red[ 2], -22], 

          Power[ 

            Red[ 3], -4], 

          Power[ 

            Red[ 5], -2], 

          Power[ 

            Red[ 7], -2]], 0, 

        Times[ 

          Power[ 

            Red[ 2], -30], 

          Power[ 

            Red[ 3], -4], 

          Power[ 

            Red[ 5], -2], 

          Power[ 

            Red[ 7], -2]]}, 0, 18, 1]]], "Output",

  CellLabel->"Out[99]="]

}, Open  ]],



Cell["\<\

However, in this paper we will concentrate on the use of colour to manipulate \

expressions.\

\>", "Text"]

}, Open  ]],



Cell[CellGroupData[{



Cell["Colour manipulation functions", "Section"],



Cell[TextData[{

  "In the following sections a number of functions for manipulating coloured \

expressions are defined. Because these functions are often used in \

combination, fully spelled out names (in the ",

  StyleBox["Mathematica",

    FontSlant->"Italic"],

  " style) were felt to be inconvenient. Each function is therefore supplied \

in a short form (which is used in the rest of the paper) as well as a fully \

spelled out form. Here is a complete table of these functions:"

}], "Text"],



Cell[BoxData[

    TagBox[GridBox[{

          {"RedOperation", "RR", 

            "\<\"Restricts functions/rules to red expressions\"\>"},

          {"RedOperationAndClear", "RR1", 

            \("Restricts functions/rules to red\nexpressions, and removes the \

red colour"\)},

          {"GreenOperation", "GG", 

            "\<\"Restricts functions/rules to green expressions\"\>"},

          {"GreenOperationAndClear", "GG1", 

            \("Restricts functions/rules to green\nexpressions, and removes \

the green colour"\)},

          {"BlueOperation", "BB", 

            "\<\"Restricts functions/rules to blue expressions\"\>"},

          {"BlueOperationAndClear", "BB1", 

            \("Restricts functions/rules to blue\nexpressions, and removes \

the blue colour"\)},

          {"CoalesceRed", "RC", 

            "\<\"Coalesces red parts of an expression\"\>"},

          {"CoalesceRedAndClear", "RC1", 

            \("Coalesces red parts of an expression\nand removes the red \

colour"\)},

          {"CoalesceGreen", "GC", 

            "\<\"Coalesces green parts of an expression\"\>"},

          {"CoalesceGreenAndClear", "GC1", 

            \("Coalesces green parts of an expression\nand removes the green \

colour"\)},

          {"CoalesceRed", "BC", 

            "\<\"Coalesces blue parts of an expression\"\>"},

          {"CoalesceBlueAndClear", "BC1", 

            \("Coalesces blue parts of an expression\nand removes the blue \

colour"\)},

          {"MoveRedOut", "Rout", 

            "\<\"Moves red expression out one layer\"\>"},

          {"MoveRedOutAndClear", "Rout1", 

            \("Moves red expression out one layer\nand removes the red \

colour"\)},

          {"MoveGreenOut", "Gout", 

            "\<\"Moves green expression out one layer\"\>"},

          {"MoveGreenOutAndClear", "Gout1", 

            \("Moves green expression out one layer\nand removes the green \

colour"\)},

          {"MoveBlueOut", "Bout", 

            "\<\"Moves blue expression out one layer\"\>"},

          {"MoveBlueOutAndClear", "Bout1", 

            \("Moves blue expression out one layer\nand removes the blue \

colour"\)},

          {"MoveRedIn", "Rin", "\<\"Moves red expression in one layer\"\>"},

          {"MoveRedInAndClear", "Rin1", 

            \("Moves red expression in one layer\nand removes the red colour"\

\)},

          {"MoveGreenIn", "Gin", 

            "\<\"Moves green expression in one layer\"\>"},

          {"MoveGreenInAndClear", "Gin1", 

            \("Moves green expression in one layer\nand removes the green \

colour"\)},

          {"MoveBlueIn", "Bin", 

            "\<\"Moves blue expression in one layer\"\>"},

          {"MoveBlueInAndClear", "Bin1", 

            \("Moves blue expression in one layer\nand removes the blue \

colour"\)},

          {"MoveRedInsideRed", "RRin", 

            "\<\"Moves red expression inside another\"\>"},

          {"MoveRedInsideRedAndClear", "RRin1", 

            \("Moves red expression inside another\nand removes the red \

colour"\)},

          {"MoveRedInsideGreen", "RGin", 

            "\<\"Moves red expression inside a green one\"\>"},

          {"MoveRedInsideGreenAndClear", "RGin1", 

            \("Moves red expression inside a green one\nand removes the red \

colour"\)},

          {"MoveRedInsideBlue", "RBin", 

            "\<\"Moves red expression inside a blue one\"\>"},

          {"MoveRedInsideBlueAndClear", "RBin1", 

            \("Moves red expression inside a blue one\nand removes the red \

colour"\)},

          {"MoveGreenInsideRed", "GRin", 

            "\<\"Moves green expression inside a red one\"\>"},

          {"MoveGreenInsideRedAndClear", "GRin1", 

            \("Moves green expression inside a red one\nand removes the green \

colour"\)},

          {"MoveGreenInsideGreen", "GGin", 

            "\<\"Moves green expression inside another\"\>"},

          {"MoveGreenInsideGreendAndClear", "GGin1", 

            \("Moves green expression inside another\nand removes the green \

colour"\)},

          {"MoveGreenInsideBlue", "GBin", 

            "\<\"Moves green expression inside a blue one\"\>"},

          {"MoveGreenInsideBlueAndClear", "GBin1", 

            \("Moves green expression inside a blue one\nand removes the \

green colour"\)},

          {"MoveBlueInsideRed", "BRin", 

            "\<\"Moves blue expression inside a red one\"\>"},

          {"MoveBlueInsideRedAndClear", "BRin1", 

            \("Moves blue expression inside a red one\nand removes the blue \

colour"\)},

          {"MoveBlueInsideGreen", "BGin", 

            "\<\"Moves blue expression inside a green one\"\>"},

          {"MoveBlueInsideGreenAndClear", "BGin1", 

            \("Moves blue expression inside a green one\nand removes the blue \

colour"\)},

          {"MoveBlueInsideBlue", "BBin", 

            "\<\"Moves blue expression inside another\"\>"},

          {"MoveBlueInsideBlueAndClear", "BBin1", 

            \("Moves blue expression inside another\nand removes the blue \

colour"\)},

          {"MoveRedOver", "Rover", "\<\"Moves red expression over ==\"\>"},

          {"MoveGreenOver", "Gover", 

            "\<\"Moves green expression over ==\"\>"},

          {"MoveBlueOver", "Bover", "\<\"Moves blue expression over ==\"\>"},

          {"MoveRedOverAndClear", "Rover1", 

            \("Moves red expression over ==\nand removes the red colour"\)},

          {"MoveGreenOverAndClear", "Gover1", 

            \("Moves green expression over ==\nand removes the green \

colour"\)},

          {"MoveBlueOverAndClear", "Bover1", 

            \("Moves blue expression over ==\nand removes the blue colour"\)}

          },

        RowSpacings->1,

        ColumnSpacings->3,

        RowAlignments->Baseline,

        ColumnAlignments->{Left}],

      (TableForm[ #]&)]], "Output",

  CellLabel->"Out[49]//TableForm="]

}, Open  ]],



Cell[CellGroupData[{



Cell["Colouring transform rules and functions", "Section"],



Cell["\<\

Users often have functions or (sets of) transformation rules set up without \

regard to issues of colour. For example, when manipulating special functions \

it is often convenient to represent them in a form that Mathematica does not \

recognise (so that unwanted evaluations do not take place) and then transform \

them using a rule such as:\

\>", "Text"],



Cell[BoxData[

    \(Myrule\  := 

      \ {\[CapitalGamma][x_] -> Gamma[x], \[Zeta][x_] -> Zeta[x]}\)], "Input"],



Cell["\<\

Clearly it is useful to have a notation which restricts the operation of a \

function or rule to operate only on coloured text without referring \

explicitly to the three colour functions. To achieve this, the functions \

RR[], GG[], BB[] are defined in the colouring package. These functions take a \

rule of function as argument and return a function which applies the argument \

only to the appropriately coloured sub-expression(s). For example, using the \

variable 'Myrule' defined above we have:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{\(\[CapitalGamma][a + b]\), "+", 

        TagBox[

          StyleBox[\(2\ \[CapitalGamma][c + d]\),

            FontColor->RGBColor[1, 0, 0]],

          Red]}], "//", " ", \(RR[Myrule]\)}]], "Input",

  CellLabel->"In[4]:="],



Cell[BoxData[

    RowBox[{

      TagBox[

        StyleBox[\(2\ Gamma[c + d]\),

          FontColor->RGBColor[1, 0, 0]],

        Red], "+", \(\[CapitalGamma][a + b]\)}]], "Output",

  CellLabel->"Out[4]="]

}, Open  ]],



Cell["\<\

These functions can accept any number of arguments, which are applied in \

turn. For example:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{\(Cos[a\ x]\), "+", 

        TagBox[

          StyleBox[\(Cos[b\ x]\),

            FontColor->RGBColor[0, 0, 1]],

          Blue], "+", \(Cos[c\ x]\)}], "//", \(BB[TrigToExp, x -> y]\)}]], 

  "Input",

  CellLabel->"In[13]:="],



Cell[BoxData[

    RowBox[{

      TagBox[

        StyleBox[\(1\/2\ \((E\^\(\(-I\)\ b\ y\) + E\^\(I\ b\ y\))\)\),

          FontColor->RGBColor[0, 0, 1]],

        Blue], "+", \(Cos[a\ x]\), "+", \(Cos[c\ x]\)}]], "Output",

  CellLabel->"Out[13]="]

}, Open  ]],



Cell["\<\

The names of all the colour manipulation functions have been kept \

deliberately short because they are typically used in combination and this \

helps to avoid cluttering expressions. \

\>", "Text"],



Cell["\<\



Note carefully that any of these six functions can be applied to rules, rule \

sets, or one-argument functions and always return a one-argument function \

which is usually applied using the // operator. Although it may seem unusual \

for a function to return another (pure) function, pure functions are just \

Mathematica expressions like any other, and in fact this technique has many \

powerful applications. 



Each colour manipulation function has a non-colour preserving (NCP) variant \

with a '1' on the end of the name. This variant removes the colour at the end \

of the process. For example:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{\(Cos[a\ x]\), "+", 

        TagBox[

          StyleBox[\(Cos[b\ x]\),

            FontColor->RGBColor[0, 0, 1]],

          Blue], "+", \(Cos[c\ x]\)}], "//", \(BB1[TrigToExp, x -> y]\)}]], 

  "Input",

  CellLabel->"In[14]:="],



Cell[BoxData[

    \(1\/2\ \((E\^\(\(-I\)\ b\ y\) + E\^\(I\ b\ y\))\) + Cos[a\ x] + 

      Cos[c\ x]\)], "Output",

  CellLabel->"Out[14]="]

}, Open  ]],



Cell["\<\

The NCP functions can also be used with no arguments to strip a colour out of \

an expression: \

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{"a", "+", 

        TagBox[

          StyleBox["b",

            FontColor->RGBColor[1, 0, 0]],

          Red], "+", 

        TagBox[

          StyleBox["a",

            FontColor->RGBColor[0, 1, 0]],

          Green], "+", 

        RowBox[{"Cos", "[", 

          TagBox[

            StyleBox["\[Theta]",

              FontColor->RGBColor[1, 0, 0]],

            Red], "]"}]}], "//", \(RR1[]\)}]], "Input",

  CellLabel->"In[4]:="],



Cell[BoxData[

    RowBox[{"a", "+", "b", "+", \(Cos[\[Theta]]\), "+", 

      TagBox[

        StyleBox["a",

          FontColor->RGBColor[0, 1, 0]],

        Green]}]], "Output",

  CellLabel->"Out[4]="]

}, Open  ]],



Cell["\<\

Here is an example in which only one of the factorial functions is converted \

to a gamma function: \

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      FractionBox[

        RowBox[{\(a!\), " ", 

          TagBox[

            StyleBox[\(b!\),

              FontColor->RGBColor[0, 0, 1]],

            Blue]}], \(c!\)], "//", \(BB1[\(x_!\)\  -> \ Gamma[x + 1]]\)}]], 

  "Input",

  CellLabel->"In[4]:="],



Cell[BoxData[

    \(\(\(a!\)\ Gamma[1 + b]\)\/\(c!\)\)], "Output",

  CellLabel->"Out[4]="]

}, Open  ]]

}, Open  ]],



Cell[CellGroupData[{



Cell["Coalescing coloured sub-expressions", "Section"],



Cell["\<\

One of the motivations for allowing multiple parts of an expression to be \

given the same colour is that sub-expressions are not necessarily contiguous. \

For  example, a+c is a sub-expression of a+b+c but you cannot pick it out by \

mouse selection because its components are scattered. The functions RC, GC, \

BC operate on coloured expressions to combine the coloured components into \

one sub-expression. Here are a few examples:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        TagBox[

          StyleBox[\(a\^2\),

            FontColor->RGBColor[1, 0, 0]],

          Red], "+", \(c\ d\), " ", "+", " ", \(c\^2\), "+", "d", "-", 

        TagBox[

          StyleBox[\(b\^2\),

            FontColor->RGBColor[1, 0, 0]],

          Red]}], "//", "RC"}]], "Input",

  CellLabel->"In[5]:="],



Cell[BoxData[

    RowBox[{\(c\^2\), "+", "d", "+", \(c\ d\), "+", 

      RowBox[{"(", 

        TagBox[

          StyleBox[\(a\^2 - b\^2\),

            FontColor->RGBColor[1, 0, 0]],

          Red], ")"}]}]], "Output",

  CellLabel->"Out[5]="]

}, Open  ]],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{\(A\ Sin[\[Alpha]]\), "+", \(B\ Sin[\[Beta]]\), "+", 

        RowBox[{

          TagBox[

            StyleBox["C",

              FontColor->RGBColor[1, 0, 0]],

            Red], " ", \(Sin[\[Gamma]]\)}], " ", "+", " ", 

        RowBox[{"D", " ", 

          RowBox[{"Sin", "[", 

            TagBox[

              StyleBox["\[Delta]",

                FontColor->RGBColor[1, 0, 0]],

              Red], "]"}]}]}], "//", "RC"}]], "Input",

  CellLabel->"In[6]:="],



Cell[BoxData[

    RowBox[{

      RowBox[{"(", 

        TagBox[

          StyleBox[\(C\ Sin[\[Gamma]] + D\ Sin[\[Delta]]\),

            FontColor->RGBColor[1, 0, 0]],

          Red], ")"}], "+", \(A\ Sin[\[Alpha]]\), "+", 

      \(B\ Sin[\[Beta]]\)}]], "Output",

  CellLabel->"Out[6]="]

}, Open  ]],



Cell["\<\

Notice that in the last example the two coloured regions could not be said to \

represent one sub-expression. Since cases such as these usually arise as a \

result of a mistake with the mouse, the coalescing functions try to interpret \

what was intended by propagating the colour until the coloured expressions \

merge or are combined by one of the basic arithmetic operators.



Using these functions in combination it is easy to define operations on parts \

of expressions which might otherwise be hard to specify: \

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        RowBox[{

          TagBox[

            StyleBox[\(a\^2\),

              FontColor->RGBColor[1, 0, 0]],

            Red], "+", \(c\ d\), " ", "+", " ", \(c\^2\), "+", "d", "-", 

          TagBox[

            StyleBox[\(b\^2\),

              FontColor->RGBColor[1, 0, 0]],

            Red]}], "//", "RC"}], "//", \(RR[Factor]\)}]], "Input",

  CellLabel->"In[5]:="],



Cell[BoxData[

    RowBox[{\(c\^2\), "+", "d", "+", \(c\ d\), "+", 

      TagBox[

        StyleBox[\(\((a - b)\)\ \((a + b)\)\),

          FontColor->RGBColor[1, 0, 0]],

        Red]}]], "Output",

  CellLabel->"Out[5]="]

}, Open  ]]

}, Open  ]],



Cell[CellGroupData[{



Cell["Moving expressions in, out, and over", "Section"],



Cell["\<\

When performing hand calculations it is common to 'move an expression out of \

the X', where X can be a set of parentheses, an integral, a sum, logarithm, \

etc. The meaning is clear once you know the context. A set of functions are \

supplied therefore to perform this operation. Their operation will be obvious \

from a few examples:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{\(\[Sum]\+\(i = 0\)\%\[Infinity]\), 

        RowBox[{

          TagBox[

            StyleBox["b",

              FontColor->RGBColor[1, 0, 0]],

            Red], " ", \(a\_i\)}]}], "//", "Rout"}]], "Input",

  CellLabel->"In[13]:="],



Cell[BoxData[

    RowBox[{

      TagBox[

        StyleBox["b",

          FontColor->RGBColor[1, 0, 0]],

        Red], " ", \(\[Sum]\+\(i = 0\)\%\[Infinity] a\_i\)}]], "Output",

  CellLabel->"Out[13]="]

}, Open  ]],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{"Log", "[", 

        RowBox[{"a", " ", 

          TagBox[

            StyleBox["b",

              FontColor->RGBColor[0, 1, 0]],

            Green], " ", "c"}], "]"}], "//", "Gout1"}]], "Input",

  CellLabel->"In[12]:="],



Cell[BoxData[

    \(Log[b] + Log[a\ c]\)], "Output",

  CellLabel->"Out[12]="]

}, Open  ]],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        RowBox[{"a", " ", 

          TagBox[

            StyleBox["x",

              FontColor->RGBColor[1, 0, 0]],

            Red]}], " ", "+", "b", " ", "+", "c"}], "//", "Rout"}]], "Input",

  CellLabel->"In[8]:="],



Cell[BoxData[

    RowBox[{"RRo", "[", 

      RowBox[{"b", "+", "c", "+", 

        RowBox[{"a", " ", 

          TagBox[

            StyleBox["x",

              FontColor->RGBColor[1, 0, 0]],

            Red]}]}], "]"}]], "Output",

  CellLabel->"Out[8]="]

}, Open  ]],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      SuperscriptBox["E", 

        RowBox[{\(a\ X\^2\), "+", \(b\ x\), "+", 

          TagBox[

            StyleBox["c",

              FontColor->RGBColor[1, 0, 0]],

            Red]}]], "//", "Rout"}]], "Input",

  CellLabel->"In[1]:="],



Cell[BoxData[

    RowBox[{\(E\^\(b\ x + a\ X\^2\)\), " ", 

      TagBox[

        StyleBox[\(E\^c\),

          FontColor->RGBColor[1, 0, 0]],

        Red]}]], "Output",

  CellLabel->"Out[1]="]

}, Open  ]],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      TagBox[

        TagBox[

          RowBox[{

            TagBox[

              StyleBox[

                RowBox[{

                  RowBox[{"a", " ", 

                    TagBox[

                      StyleBox["x",

                        FontColor->RGBColor[1, 0, 0]],

                      Red]}], "+", "b"}],

                FontColor->RGBColor[0, 0, 1]],

              Blue], "+", "c"}],

          Blue],

        Blue], "//", "Rout"}]], "Input",

  CellLabel->"In[10]:="],



Cell[BoxData[

    TagBox[

      StyleBox[

        TagBox[

          StyleBox[

            RowBox[{"c", "+", 

              TagBox[

                StyleBox[

                  RowBox[{\((a + b\/x)\), " ", 

                    TagBox[

                      StyleBox["x",

                        FontColor->RGBColor[1, 0, 0]],

                      Red]}],

                  FontColor->RGBColor[0, 0, 1]],

                Blue]}],

            FontColor->RGBColor[0, 0, 1]],

          Blue],

        FontColor->RGBColor[0, 0, 1]],

      Blue]], "Output",

  CellLabel->"Out[10]="]

}, Open  ]],



Cell["\<\

The functions RRout, GGout, BBout (plus their NCP variants) pull a coloured \

expression out of whatever it is embedded in. Note that these functions do \

not check that for example a coloured expression is independent of the \

integration variable before extracting it from an integral - this is left up \

to the user at present.

The last example is particularly instructive. Pulling x out of the sum a x +b \

+c would produce (a+b/x+c/x) x, however by using another colour to mark part \

of the sum we can be more selective. Remember that by colouring part of the \

expression blue we have isolated it by wrapping it in a function call. \

\>", "Text"],



Cell["\<\

While the process of moving an expression out is usually well defined, moving \

an expression in is frequently not well defined. Consider the meaning of \

moving the red expression in in each of the following examples:\

\>", "Text"],



Cell[BoxData[

    RowBox[{

      TagBox[

        StyleBox["a",

          FontColor->RGBColor[1, 0, 0]],

        Red], " ", \((b + c)\)}]], "Input"],



Cell[BoxData[

    RowBox[{

      TagBox[

        StyleBox["a",

          FontColor->RGBColor[1, 0, 0]],

        Red], \((b + c)\), \((d + e)\)}]], "Input"],



Cell[BoxData[

    RowBox[{

      RowBox[{"(", 

        TagBox[

          StyleBox[\(a + b\),

            FontColor->RGBColor[1, 0, 0]],

          Red], ")"}], \((c + d)\)}]], "Input"],



Cell["\<\

While moving the red expression inwards is well defined in the first example, \

this is not the case in the second. We could solve the ambiguity involved in \

the second case by requiring both expressions to be given the same colour, \

however the third example illustrates that even this is not necessarily \

adequate, because moving (a+b) into (c+d) gives c(a+b)+d(a+b), whereas moving \

(c+d) into (a+b) produces a(c+d)+b(c+d). To adequately specify a 'moving in' \

operation we therefore need two colours, although the same colour can be used \

where no ambiguity arises. The three functions Rin, Gin, Bin can be used \

where no ambiguity arises, and nine more functions handle the general case. \

For example, the function RGi takes a red expression inside a green one. Here \

are some examples:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        RowBox[{"(", 

          TagBox[

            StyleBox[\(a + b\),

              FontColor->RGBColor[1, 0, 0]],

            Red], ")"}], \((c + d)\), 

        RowBox[{"(", 

          TagBox[

            StyleBox[\(e + f\),

              FontColor->RGBColor[0, 1, 0]],

            Green], ")"}]}], "//", "RGin"}]], "Input",

  CellLabel->"In[12]:="],



Cell[BoxData[

    RowBox[{\((c + d)\), " ", 

      RowBox[{"(", 

        RowBox[{

          RowBox[{"e", " ", 

            RowBox[{"(", 

              TagBox[

                StyleBox[\(a + b\),

                  FontColor->RGBColor[1, 0, 0]],

                Red], ")"}]}], "+", 

          RowBox[{"f", " ", 

            RowBox[{"(", 

              TagBox[

                StyleBox[\(a + b\),

                  FontColor->RGBColor[1, 0, 0]],

                Red], ")"}]}]}], ")"}]}]], "Output",

  CellLabel->"Out[12]="]

}, Open  ]],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        TagBox[

          StyleBox[\(g[a]\),

            FontColor->RGBColor[1, 0, 0]],

          Red], " ", \((c - d)\), " ", 

        TagBox[

          StyleBox[\(\[Sum]\+\(i = 0\)\%\[Infinity]f[i]\),

            FontColor->RGBColor[0, 1, 0]],

          Green]}], " ", "//", " ", "RGin"}]], "Input",

  CellLabel->"In[18]:="],



Cell[BoxData[

    RowBox[{\((c - d)\), " ", 

      RowBox[{\(\[Sum]\+\(i = 0\)\%\[Infinity]\), 

        RowBox[{

          TagBox[

            StyleBox[\(g[a]\),

              FontColor->RGBColor[1, 0, 0]],

            Red], " ", \(f[i]\)}]}]}]], "Output",

  CellLabel->"Out[18]="]

}, Open  ]],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{\(\[CapitalGamma][1 - z]\ \[CapitalGamma][z]\), "==", 

        RowBox[{"\[Pi]", " ", 

          TagBox[

            StyleBox[\(Csc[\[Pi]\ z]\),

              FontColor->RGBColor[1, 0, 0]],

            Red]}]}], "//", "Rover1"}]], "Input",

  CellLabel->"In[13]:="],



Cell[BoxData[

    \(Sin[\[Pi]\ z]\ \[CapitalGamma][1 - z]\ \[CapitalGamma][z] == \[Pi]\)], 

  "Output",

  CellLabel->"Out[13]="]

}, Open  ]],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        RowBox[{

          TagBox[

            StyleBox[\(Sin[\[Pi]\ z]\),

              FontColor->RGBColor[1, 0, 0]],

            Red], " ", \(\[CapitalGamma][1 - z]\), " ", 

          \(\[CapitalGamma][z]\)}], "==", "\[Pi]"}], "//", "Rover"}]], "Input",\



  CellLabel->"In[14]:="],



Cell[BoxData[

    RowBox[{\(\[CapitalGamma][1 - z]\ \[CapitalGamma][z]\), "==", 

      FractionBox["\[Pi]", 

        TagBox[

          StyleBox[\(Sin[\[Pi]\ z]\),

            FontColor->RGBColor[1, 0, 0]],

          Red]]}]], "Output",

  CellLabel->"Out[14]="]

}, Open  ]],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        RowBox[{\(a\^2\), "+", \(b\^2\), "+", 

          TagBox[

            StyleBox[\(Log[a + b]\),

              FontColor->RGBColor[1, 0, 0]],

            Red], "+", 

          TagBox[

            StyleBox[\(Log[c + d]\),

              FontColor->RGBColor[0, 1, 0]],

            Green], "-", 

          TagBox[

            StyleBox[\(Log[a - b]\),

              FontColor->RGBColor[1, 0, 0]],

            Red], "-", 

          TagBox[

            StyleBox[\(Log[c - d]\),

              FontColor->RGBColor[0, 1, 0]],

            Green]}], "//", "RRin1"}], "//", "GGin1"}]], "Input",

  CellLabel->"In[2]:="],



Cell[BoxData[

    \(a\^2 + b\^2 + Log[\(a + b\)\/\(a - b\)] + Log[\(c + d\)\/\(c - d\)]\)], 

  "Output",

  CellLabel->"Out[2]="]

}, Open  ]],



Cell[TextData[{

  "Notice how the gamma function identity is re-arranged to get rid of the \

Csc function. The denominator has to remain coloured to prevent ",

  StyleBox["Mathematica",

    FontSlant->"Italic"],

  " 'simplifying' the result by using Csc."

}], "Text"],



Cell["\<\

The last example illustrates a case where the same colour can be used without \

ambiguity (there is only one way to combine the difference of two logarithms) \

and represents the neatest way to transform this expression using colour. By \

using the NCP variants the final operation of removing the colour has also \

been achieved.\

\>", "Text"],



Cell["\<\

Coloured expressions can also be used to manipulate equalities (specified by \

the == operator) and inequalities (specified by >, >=, etc.) to achieve \

results not easily obtained by other means (such as the 'Solve' function). \

Three functions (and NCP variants) are available to move expressions to the \

oposite side of an == or inequality operator by dividing or subtraction as \

appropriate. For example:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        RowBox[{"a", "+", 

          TagBox[

            StyleBox[\(b + c\),

              FontColor->RGBColor[1, 0, 0]],

            Red], "+", "d"}], "==", "0"}], "//", "Rover1"}]], "Input",

  CellLabel->"In[44]:="],



Cell[BoxData[

    \(a + d == \(-b\) - c\)], "Output",

  CellLabel->"Out[44]="]

}, Open  ]],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        RowBox[{"x", "+", 

          RowBox[{

            TagBox[

              StyleBox["a",

                FontColor->RGBColor[1, 0, 0]],

              Red], " ", "b"}]}], "==", "d"}], " ", "//", "Rover"}]], "Input",\



  CellLabel->"In[46]:="],



Cell[BoxData[

    RowBox[{

      RowBox[{"b", "+", 

        FractionBox["x", 

          TagBox[

            StyleBox["a",

              FontColor->RGBColor[1, 0, 0]],

            Red]]}], "==", 

      FractionBox["d", 

        TagBox[

          StyleBox["a",

            FontColor->RGBColor[1, 0, 0]],

          Red]]}]], "Output",

  CellLabel->"Out[46]="]

}, Open  ]],



Cell["\<\

It is important to realise that it is the user's responsibility to ensure \

that these operations make sense mathematically. For example it is wrong to \

divide both sides of an equation by something which ultimately turns out to \

be zero. Similarly it is in general wrong to divide both sides of an equality \

by something which turns out to be negative. Nevertheless, there is much less \

scope for algebraic errors using these operations than by using complex \

transformation rules.\

\>", "Text"]

}, Open  ]],



Cell[CellGroupData[{



Cell["Protected mathematical operations", "Section"],



Cell[TextData[{

  "One weakness of the current version of ",

  StyleBox["Mathematica",

    FontSlant->"Italic"],

  " is that it lacks any way of describing sums, integrals, limits and \

products without inviting the system to evaluate them. Consider for example \

the following sum:"

}], "Text"],



Cell[BoxData[

    \(\[Sum]\+\(k = 1\)\%1000 a\_k\)], "Input",

  CellLabel->"In[31]:="],



Cell[TextData[{

  "This sum will be immediately 'evaluated' to produce an expression with \

1000 terms. To avoid this the package defines ",

  StyleBox["protected",

    FontVariations->{"Underline"->True}],

  " versions of all these operations. All four are available from the palette \

and map the underlying operations as follows:\n\nSum->Summation\n\

Product->Product1\nIntegrate->Integral\nLimit->Limit1\n\nEach operation \

displays in StandardForm using a slightly thicker font than the normal \

operation. All the colour operations (such as Rin) can be used to move things \

in and out of these operations. Also, the following functions transform the \

protected forms into the active operations:\n\nDoSummations\nDoProducts\n\

DoLimits\nDoIntegrals\n"

}], "Text"],



Cell["\<\

Here is a simple example of a colour operation being performed inside a \

protected integral.\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        SubsuperscriptBox[

          StyleBox["\[Integral]",

            FontFamily->"Arial Black"], "0", \(1/2\)], 

        RowBox[{

          SqrtBox[

            RowBox[{"-", 

              FractionBox[

                RowBox[{"1", "+", 

                  TagBox[

                    StyleBox[\(k\^2\ \((\(-1\) + v\^2)\)\),

                      FontColor->RGBColor[1, 0, 0]],

                    Red]}], 

                \(\((\(-1\) + k\^2)\)\^2\ \((\(-1\) + v\^2)\)\)]}]], 

          RowBox[{

            TagBox["\[DifferentialD]",

              Integral], "v"}]}]}], "//", \(RR1[Expand]\)}]], "Input",

  CellLabel->"In[35]:="],



Cell[BoxData[

    RowBox[{

      SubsuperscriptBox[

        StyleBox["\[Integral]",

          FontFamily->"Arial Black"], "0", \(1\/2\)], 

      RowBox[{

      \(\@\(-\(\(1 - k\^2 + k\^2\ v\^2\)\/\(\((\(-1\) + k\^2)\)\^2\ 

                    \((\(-1\) + v\^2)\)\)\)\)\), 

        RowBox[{

          TagBox["\[DifferentialD]",

            Integral], "v"}]}]}]], "Output",

  CellLabel->"Out[35]="]

}, Open  ]],



Cell["\<\

Despite its complicated appearance, this integral would be immediately \

evaluated by the normal integral operator, so that manipulation of the \

integrand would not be possible:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        SubsuperscriptBox[

          StyleBox["\[Integral]",

            FontFamily->"Arial Black"], "0", \(1/2\)], 

        RowBox[{

        \(\@\(-\(\(1 + k\^2\ \((\(-1\) + v\^2)\)\)\/\(\((\(-1\) + k\^2)\)\^2\ 

                      \((\(-1\) + v\^2)\)\)\)\)\), 

          RowBox[{

            TagBox["\[DifferentialD]",

              Integral], "v"}]}]}], "//", "DoIntegrals"}]], "Input",

  CellLabel->"In[34]:="],



Cell[BoxData[

    \(\(\@\(\(1 - \(3\ k\^2\)\/4\)\/\((\(-1\) + k\^2)\)\^2\)\ 

        EllipticE[\[Pi]\/6, k\^2\/\(\(-1\) + k\^2\)]

          \)\/\@\(\(\(-1\) + \(3\ k\^2\)\/4\)\/\(\(-1\) + k\^2\)\)\)], 

  "Output",

  CellLabel->"Out[34]="]

}, Open  ]],



Cell["\<\

Protected mathematical operations offer scope for a wide variety of \

additional useful transformations, such as change of integration variable, \

combining and/or shifting sums and products, etc. Many of these have been \

indeed been defined, but are not described here.\

\>", "Text"]

}, Open  ]],



Cell[CellGroupData[{



Cell["Inserting expressions", "Section"],



Cell[TextData[{

  "A further common trick in hand calculations is to insert a term on the top \

and bottom of an expression, or to add and subtract a term. Although the \

result is more complex than before, the idea is to facilitate further \

operations. Without colour such manipulations are not normally possible with \

",

  StyleBox["Mathematica",

    FontSlant->"Italic"],

  " . For example, multiplying an expression by ",

  Cell[BoxData[

      \(TraditionalForm\`\(2\^n/2\^n\ \)\)]],

  "would produce immediate cancelation. The colour operations Rmpy,Gmpy,Bmpy \

and the corresponding Radd,Gadd,Badd solve this problem. Rmpy, for example, \

finds a red expression, removes the colour, multiplies it with a supplied \

expression, and divides by a red version of the same expression. Here is an \

example using protected products:\n"

}], "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        RowBox[{

          UnderoverscriptBox[

            StyleBox["\[Product]",

              FontFamily->"Arial Black"], 

            RowBox[{"k", 

              TagBox["=",

                Product1], "1"}], "n"], 

          RowBox[{"(", " ", 

            TagBox[

              StyleBox[\(\(-\(1\/2\)\) + k\),

                FontColor->RGBColor[1, 0, 0]],

              Red], ")"}]}], "//", " ", \(Rmpy[2]\)}], "//", "Rout1"}]], 

  "Input",

  CellLabel->"In[40]:="],



Cell[BoxData[

    RowBox[{\(2\^\(-n\)\), " ", 

      TagBox[

        RowBox[{

          UnderoverscriptBox[

            StyleBox["\[Product]",

              FontFamily->"Arial Black"], 

            RowBox[{"k", 

              TagBox["=",

                Product1], "1"}], "n"], "(", \(2\ \((\(-\(1\/2\)\) + k)\)\), 

          ")"}],

        Product1]}]], "Output",

  CellLabel->"Out[40]="]

}, Open  ]],



Cell["\<\

Once again, without the protected operation this manipulation is not \

possible:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{\(2\^\(-n\)\), " ", 

        TagBox[

          RowBox[{

            UnderoverscriptBox[

              StyleBox["\[Product]",

                FontFamily->"Arial Black"], 

              RowBox[{"k", 

                TagBox["=",

                  Product1], "1"}], "n"], "(", 

            \(2\ \((\(-\(1\/2\)\) + k)\)\), ")"}],

          Product1]}], "//", "DoProducts"}]], "Input",

  CellLabel->"In[41]:="],



Cell[BoxData[

    \(Gamma[1\/2 + n]\/\@\[Pi]\)], "Output",

  CellLabel->"Out[41]="]

}, Open  ]],



Cell["\<\

Notice how because one component of the insertion is coloured, it can be \

immediately subjected to a further operation. \

\>", "Text"]

}, Open  ]],



Cell[CellGroupData[{



Cell["Some more substantial examples", "Section"],



Cell["\<\

The following problem comes from CLM. The user wished to transform the \

following expression so as to group the terms with the same dependence on \

y:\

\>", "Text"],



Cell[BoxData[

    RowBox[{

      TagBox[

        \(c\/y\^2 + d\/y\^2 + e\/y\^0.1` + f\/y\^0.1` + a\ y + b\ y + 

          \(g\ Log[y]\)\/y - \(h\ Log[y]\)\/y\),

        Red], 

      TagBox["",

        Green]}]], "Input"],



Cell["\<\

Despite the fact that the required operation is fairly trivial, the proposed \

solution required the replacement of the exponent 0.1 by 1/10 and a fairly \

complicated manipulation using the Collect function. Using colour we can \

simply pick off sub-expressions and factorise them:\

\>", "Text"],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

        RowBox[{

          RowBox[{

            TagBox[

              StyleBox[\(a\ y + b\ y\),

                FontColor->RGBColor[1, 0, 0]],

              Red], "+", 

            TagBox[

              StyleBox[\(c\/y\^2 + d\/y\^2\),

                FontColor->RGBColor[0, 1, 0]],

              Green], "+", 

            TagBox[

              StyleBox[

                RowBox[{

                  FractionBox["e", 

                    SuperscriptBox["y", 

                      StyleBox["0.1`",

                        StyleBoxAutoDelete->True,

                        PrintPrecision->1]]], "+", 

                  FractionBox["f", 

                    SuperscriptBox["y", 

                      StyleBox["0.1`",

                        StyleBoxAutoDelete->True,

                        PrintPrecision->1]]]}],

                FontColor->RGBColor[0, 0, 1]],

              Blue], "+", \(g\ Log[y]/y\), "-", \(h\ Log[y]/y\)}], "//", 

          \(RR1[Factor]\)}], "//", \(GG1[Factor]\)}], "//", 

      \(BB1[Factor]\)}]], "Input",

  CellLabel->"In[25]:="],



Cell[BoxData[

    \(\(c + d\)\/y\^2 + \(e + f\)\/y\^0.1` + \((a + b)\)\ y + 

      \(g\ Log[y]\)\/y - \(h\ Log[y]\)\/y\)], "Output",

  CellLabel->"Out[25]="]

}, Open  ]],



Cell[CellGroupData[{



Cell[BoxData[

    RowBox[{

      RowBox[{

      \(\(c + d\)\/y\^2\), "+", \(\(e + f\)\/y\^0.1`\), "+", 

        \(\((a + b)\)\ y\), "+", 

        TagBox[

          StyleBox[\(\(g\ Log[y]\)\/y - \(h\ Log[y]\)\/y\),

            FontColor->RGBColor[1, 0, 0]],

          Red]}], "//", \(RR1[Factor]\)}]], "Input",

  CellLabel->"In[26]:="],



Cell[BoxData[

    \(\(c + d\)\/y\^2 + \(e + f\)\/y\^0.1` + \((a + b)\)\ y + 

      \(\((g - h)\)\ Log[y]\)\/y\)], "Output",

  CellLabel->"Out[26]="]

}, Open  ]],



Cell["\<\

If the coloured expression package catered for four colours this operation \

would, of course, have been complete in one step. \

\>", "Text"]

}, Open  ]],



Cell[CellGroupData[{



Cell["A final touch", "Section"],



Cell["\<\

When performing hand calculations it is often useful to extract a \

sub-expression, manipulate the expression for a while on its own, before \

re-inserting it into the full equation. To achieve this, three buttons are \

provided (for red,green, and blue) that extract a coloured expression \

(applying colour coalescence if several parts are coloured) and present the \

expression in a temporary notebook. A button is provided in the temporary \

notebook to re-insert the result (destroying the temporary notebook and \

returning the user to the original notebook in the process).\

\>", "Text"]

}, Open  ]],



Cell[CellGroupData[{



Cell["Implementation of frontend-kernel transformations", "Section"],



Cell["\<\

As indicated above, Mathematica is set up so as to ignore colour distinctions \

within expressions. While it would have been possible to add rules to \

MakeExpression to change this behaviour, it was decided that a cleaner \

approach was to apply a Red,Green, or Blue tag to coloured sub-expressions, \

which automatically gets converted to the corresponding function application \

by MakeExpression. Thus when the red button is applied to the variable 'a' it \

splices in the following boxes: \

\>", "Text"],



Cell[BoxData[

    TagBox[\(TagBox[StyleBox["a", FontColor -> RGBColor[1, 0, 0]], Red]\),

      Red]], "Input",

  CellLabel->"In[4]:="],



Cell["\<\



The splicing action of the button is not quite standard and is therefore \

implemented by a private function in the package. This function checks that \

the selected text constitutes a valid sub-expression and also removes any \

existing colour from within it.\

\>", "Text"],



Cell["\<\



The MakeBoxes function (which performs the transformation between Kernel \

format and the 'Box' format, used by the frontend) is augmented by the \

package to cope with the 'Red' function thus:\

\>", "Text"],



Cell[BoxData[

    \(Red /: \ MakeBoxes[Red[x_], StandardForm] := \n

      \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 

      TagBox[\n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 

        StyleBox[MakeBoxes[x, StandardForm], FontColor -> RGBColor[1, 0, 0]]\n

        \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ , Red]

        \)], "Input",

  InitializationCell->True],



Cell["\<\



The other two colours are treated analogously. The expression tag ensures \

that MakeExpression will insert the necessary colour functions as it converts \

back to Kernel form - the actual colour is ignored in this process.\

\>", "Text"]

}, Open  ]],



Cell[CellGroupData[{



Cell["Implementation of the RR functions.", "Section"],



Cell["\<\

All the functions for specialising rules and functions are channeled through \

the 'colourtransform' function, which is private to the package.  \

\>", "Text"],



Cell[BoxData[

    \(\(RR[x___] := colourtransform[Red, Flatten[{x}], Red]; \)\)], "Input"],



Cell["\<\

This definition has the effect that multiple arguments to RR and/or lists of \

transformation rules all get flattened into a simple list. The software then \

starts with an identity function of the form:\

\>", "Text"],



Cell[BoxData[

    \(Function[x, x]\)], "Input"],



Cell["\<\

Each argument of the list of functions/transformations is then spliced into \

the right hand side of this object. Consider for example:\

\>", "Text"],



Cell[BoxData[

    \(RR[a -> b]\)], "Input",

  CellLabel->"In[5]:="],



Cell["\<\

The result of evaluating this function is hard to read because the variables \

are all private to the coloured expression package, and also because \

references to the 'Red' function are converted into coloured text! Here is a \

translation of the result:\

\>", "Text"],



Cell[BoxData[

    RowBox[{

      RowBox[{"Function", "[", 

        RowBox[{"Z", ",", 

          RowBox[{"Z", "/.", 

            RowBox[{

              RowBox[{"Red", "[", "x_", 

                TagBox["]",

                  Red]}], "\[RuleDelayed]", 

              TagBox[\(Red[x /. \ a -> b\),

                Red]}]}]}], "]"}], "]"}]], "Input",

  CellLabel->"Out[5]="],



Cell["\<\

The last argument to colourtransform determines the final colour of coloured \

expressions. RR1 sets this to Identity - the Mathematica function that simply \

returns its argument unchanged.\

\>", "Text"]

}, Open  ]],



Cell[CellGroupData[{



Cell["\<\

Implementing expression coalescing and the other colour manipulation \

functions\

\>", "Section"],



Cell["\<\

The RC,GC, BC functions (and their NCP variants) are all channeled through a \

single function colourmerge:\

\>", "Text"],



Cell[BoxData[

    \(\(RC[x_] := colourmerge[Red, x]; \)\)], "Input"],



Cell["\<\

The implementation of the colourmerge function is a little subtle because the \

changes required at any part of the expression potentially depend on the \

whole of the rest of the expression. The function is implemented recursively \

and each invokation is passed a flag variable which it can set to True if the \

expression it returns contains the colour in question. The Plus and Times \

records (which include subtraction and division because of the structure of \

Mathematica Kernel syntax) are treated separately. The number of distinct \

instances of coloured sub expressions are counted and if there are more than \

1 then the colour is removed and re-applied round all the coloured components \

- thereby performing the desired coalescence. Here is the complete code for \

this function including three sub functions that it uses. The author makes no \

claim that this is the most elegant way to perform this operation!\

\>", "Text"],



Cell[BoxData[

    \(Clear[colourmerge, cm1, cm2, cm3]; \n

    colourmerge[C_, x_] := Module[{Flag}, colourmerge[C, x, Flag]]; \n

    colourmerge[C_, x_?AtomQ, Flag_] := \ x; \n

    colourmerge[C_, C_[x_], Flag_] := \ \((cm3[Flag]; C[x /. C[a_] -> a])\); 

    \ncolourmerge[C_, Alternatives[x_Plus, x_Times], Flag_]\  := \n\t

      Module[{Flag1, ans}, \n\tans = Map[cm1[C, #1]&, x]; \n\t\t

        Switch[Count[ans, f[True, _]], \n\t\t\t0, Map[#1[\([2]\)]&, ans], \n

          \t\t\t1, cm3[Flag]; Map[#1[\([2]\)]&, ans], \n\t\t\t\t_, cm3[Flag]; 

          \(Map[cm2[C, #1]&, ans /. C[u_] -> u] //. 

              C[a_] + C[b_] -> C[a + b]\) //. C[a_] C[b_] -> C[a\ b]\n\t]]; 

    \t\ \ncolourmerge[C_, F_[x___], Flag_] := \n\t\t

      Module[{Flag1, ans}, \n\t\t\ 

        ans = Map[colourmerge[C, #1, Flag1]&, F[x], Heads -> True]; \n\t\t

        If[Flag1, cm3[Flag]]; \n\t\tans]; \n

    cm1[C_, x_]\  := \ 

      Module[{Flag, ans}, \n\t\tans = colourmerge[C, x, Flag]; \n\t\t

        If[Flag, Return[f[True, ans]]]; \n\t\tf[False, ans]\n\t]; \n

    cm2[C_, f[True, x_]] := C[x]; \ncm2[C_, f[_, x_]] := x; \n

    cm3[True] := True; \ncm3[x_] := \((x = True)\); \)], "Input",

  CellLabel->"In[48]:=",

  InitializationCell->True],



Cell["\<\

It is important to bear in mind that this code is designed to run inside a \

package. This means that functions such as 'f' used to hold expression/flag \

pairs cannot get confused with user-defined functions of the same name.\

\>", "Text"],



Cell["\<\

The other colour manipulation functions (RRin, RRout) are handled on a case \

by case basis handling each algebraic situation that they can handle. These \

functions leave an expression unchanged (except for the removal of colour by \

the NCP variants) if no re-arrangement is possible.\

\>", "Text"]

}, Open  ]],



Cell[CellGroupData[{



Cell["Conclusion", "Section"],



Cell["\<\

Although Mathematica has been available for many years, the frontend is a \

relatively new part of the product. The fact that the relationship between \

the Kernel representation of expressions and the 2-d/coloured representation \

(known as BoxForm) is well defined and fully programmable means that the \

expression colouring technique described in this paper is probably only one \

of many possibilities waiting to be exploited. \

\>", "Text"]

}, Open  ]]

},

FrontEndVersion->"Microsoft Windows 3.0",

ScreenRectangle->{{0, 1024}, {0, 740}},

AutoGeneratedPackage->None,

WindowToolbars->"EditBar",

WindowSize->{864, 407},

WindowMargins->{{-30, Automatic}, {Automatic, 62}},

PrintingCopies->1,

PrintingPageRange->{Automatic, Automatic},

Magnification->1,

StyleDefinitions -> "ArticleClassic.nb"

]





(***********************************************************************

Cached data follows.  If you edit this Notebook file directly, not using

Mathematica, you must remove the line containing CacheID at the top of 

the file.  The cache data will then be recreated when you save this file 

from within Mathematica.

***********************************************************************)



(*CellTagsOutline

CellTagsIndex->{}

*)



(*CellTagsIndex

CellTagsIndex->{}

*)



(*NotebookFileOutline

Notebook[{

Cell[1709, 49, 137, 4, 44, "Input"],

Cell[1849, 55, 137, 5, 54, "Text"],



Cell[CellGroupData[{

Cell[2011, 64, 31, 0, 56, "Section"],

Cell[2045, 66, 1143, 20, 230, "Text"],

Cell[3191, 88, 99, 2, 36, "Input"],

Cell[3293, 92, 69, 0, 32, "Text"],

Cell[3365, 94, 113, 2, 56, "Input"],

Cell[3481, 98, 1217, 19, 318, "Text"]

}, Open  ]],



Cell[CellGroupData[{

Cell[4735, 122, 35, 0, 56, "Section"],

Cell[4773, 124, 61, 0, 32, "Text"],

Cell[4837, 126, 99, 2, 36, "Input"],

Cell[4939, 130, 263, 5, 76, "Text"],

Cell[5205, 137, 240, 6, 36, "Input"],

Cell[5448, 145, 233, 4, 54, "Text"],

Cell[5684, 151, 320, 9, 36, "Input"],

Cell[6007, 162, 107, 3, 32, "Text"],



Cell[CellGroupData[{

Cell[6139, 169, 435, 12, 62, "Input"],

Cell[6577, 183, 126, 3, 49, "Output"]

}, Open  ]],

Cell[6718, 189, 154, 3, 54, "Text"],



Cell[CellGroupData[{

Cell[6897, 196, 440, 13, 85, "Input"],

Cell[7340, 211, 128, 3, 49, "Output"]

}, Open  ]],

Cell[7483, 217, 350, 6, 76, "Text"],

Cell[7836, 225, 130, 3, 32, "Text"],



Cell[CellGroupData[{

Cell[7991, 232, 211, 7, 36, "Input"],

Cell[8205, 241, 208, 6, 35, "Output"]

}, Open  ]],

Cell[8428, 250, 860, 14, 208, "Text"],



Cell[CellGroupData[{

Cell[9313, 268, 299, 10, 36, "Input"],

Cell[9615, 280, 525, 17, 36, "Output"]

}, Open  ]],

Cell[10155, 300, 277, 7, 54, "Text"],

Cell[10435, 309, 196, 5, 88, "Input"],



Cell[CellGroupData[{

Cell[10656, 318, 137, 3, 36, "Input"],

Cell[10796, 323, 4915, 167, 54, "Output"]

}, Open  ]],

Cell[15726, 493, 116, 3, 32, "Text"]

}, Open  ]],



Cell[CellGroupData[{

Cell[15879, 501, 48, 0, 56, "Section"],

Cell[15930, 503, 496, 9, 98, "Text"],

Cell[16429, 514, 5825, 123, 1831, "Output"]

}, Open  ]],



Cell[CellGroupData[{

Cell[22291, 642, 58, 0, 56, "Section"],

Cell[22352, 644, 366, 6, 76, "Text"],

Cell[22721, 652, 111, 2, 36, "Input"],

Cell[22835, 656, 525, 8, 120, "Text"],



Cell[CellGroupData[{

Cell[23385, 668, 264, 7, 36, "Input"],

Cell[23652, 677, 203, 6, 35, "Output"]

}, Open  ]],

Cell[23870, 686, 117, 3, 32, "Text"],



Cell[CellGroupData[{

Cell[24012, 693, 266, 8, 36, "Input"],

Cell[24281, 703, 245, 6, 49, "Output"]

}, Open  ]],

Cell[24541, 712, 208, 4, 54, "Text"],

Cell[24752, 718, 624, 12, 186, "Text"],



Cell[CellGroupData[{

Cell[25401, 734, 267, 8, 36, "Input"],

Cell[25671, 744, 138, 3, 49, "Output"]

}, Open  ]],

Cell[25824, 750, 119, 3, 32, "Text"],



Cell[CellGroupData[{

Cell[25968, 757, 468, 16, 36, "Input"],

Cell[26439, 775, 200, 6, 35, "Output"]

}, Open  ]],

Cell[26654, 784, 124, 3, 32, "Text"],



Cell[CellGroupData[{

Cell[26803, 791, 280, 9, 56, "Input"],

Cell[27086, 802, 90, 2, 49, "Output"]

}, Open  ]]

}, Open  ]],



Cell[CellGroupData[{

Cell[27225, 810, 54, 0, 56, "Section"],

Cell[27282, 812, 458, 7, 98, "Text"],



Cell[CellGroupData[{

Cell[27765, 823, 355, 11, 36, "Input"],

Cell[28123, 836, 241, 7, 36, "Output"]

}, Open  ]],



Cell[CellGroupData[{

Cell[28401, 848, 499, 14, 36, "Input"],

Cell[28903, 864, 285, 8, 35, "Output"]

}, Open  ]],

Cell[29203, 875, 540, 9, 164, "Text"],



Cell[CellGroupData[{

Cell[29768, 888, 412, 12, 36, "Input"],

Cell[30183, 902, 219, 6, 36, "Output"]

}, Open  ]]

}, Open  ]],



Cell[CellGroupData[{

Cell[30451, 914, 55, 0, 56, "Section"],

Cell[30509, 916, 359, 6, 76, "Text"],



Cell[CellGroupData[{

Cell[30893, 926, 271, 8, 65, "Input"],

Cell[31167, 936, 201, 6, 65, "Output"]

}, Open  ]],



Cell[CellGroupData[{

Cell[31405, 947, 260, 8, 36, "Input"],

Cell[31668, 957, 77, 2, 35, "Output"]

}, Open  ]],



Cell[CellGroupData[{

Cell[31782, 964, 259, 8, 36, "Input"],

Cell[32044, 974, 253, 8, 35, "Output"]

}, Open  ]],



Cell[CellGroupData[{

Cell[32334, 987, 263, 8, 38, "Input"],

Cell[32600, 997, 191, 6, 38, "Output"]

}, Open  ]],



Cell[CellGroupData[{

Cell[32828, 1008, 505, 17, 36, "Input"],

Cell[33336, 1027, 575, 19, 49, "Output"]

}, Open  ]],

Cell[33926, 1049, 663, 10, 142, "Text"],

Cell[34592, 1061, 242, 4, 54, "Text"],

Cell[34837, 1067, 147, 5, 36, "Input"],

Cell[34987, 1074, 155, 5, 36, "Input"],

Cell[35145, 1081, 183, 6, 36, "Input"],

Cell[35331, 1089, 824, 12, 164, "Text"],



Cell[CellGroupData[{

Cell[36180, 1105, 394, 13, 36, "Input"],

Cell[36577, 1120, 520, 16, 35, "Output"]

}, Open  ]],



Cell[CellGroupData[{

Cell[37134, 1141, 368, 11, 68, "Input"],

Cell[37505, 1154, 281, 8, 65, "Output"]

}, Open  ]],



Cell[CellGroupData[{

Cell[37823, 1167, 304, 8, 36, "Input"],

Cell[38130, 1177, 128, 3, 35, "Output"]

}, Open  ]],



Cell[CellGroupData[{

Cell[38295, 1185, 325, 10, 36, "Input"],

Cell[38623, 1197, 260, 7, 49, "Output"]

}, Open  ]],



Cell[CellGroupData[{

Cell[38920, 1209, 652, 20, 36, "Input"],

Cell[39575, 1231, 128, 3, 49, "Output"]

}, Open  ]],

Cell[39718, 1237, 267, 6, 54, "Text"],

Cell[39988, 1245, 354, 6, 76, "Text"],

Cell[40345, 1253, 434, 7, 98, "Text"],



Cell[CellGroupData[{

Cell[40804, 1264, 259, 8, 36, "Input"],

Cell[41066, 1274, 78, 2, 35, "Output"]

}, Open  ]],



Cell[CellGroupData[{

Cell[41181, 1281, 288, 10, 36, "Input"],

Cell[41472, 1293, 356, 13, 49, "Output"]

}, Open  ]],

Cell[41843, 1309, 511, 8, 120, "Text"]

}, Open  ]],



Cell[CellGroupData[{

Cell[42391, 1322, 52, 0, 56, "Section"],

Cell[42446, 1324, 295, 7, 54, "Text"],

Cell[42744, 1333, 86, 2, 68, "Input"],

Cell[42833, 1337, 770, 13, 406, "Text"],

Cell[43606, 1352, 117, 3, 32, "Text"],



Cell[CellGroupData[{

Cell[43748, 1359, 674, 19, 87, "Input"],

Cell[44425, 1380, 396, 11, 68, "Output"]

}, Open  ]],

Cell[44836, 1394, 202, 4, 54, "Text"],



Cell[CellGroupData[{

Cell[45063, 1402, 458, 12, 87, "Input"],

Cell[45524, 1416, 238, 5, 126, "Output"]

}, Open  ]],

Cell[45777, 1424, 295, 5, 76, "Text"]

}, Open  ]],



Cell[CellGroupData[{

Cell[46109, 1434, 40, 0, 56, "Section"],

Cell[46152, 1436, 849, 16, 186, "Text"],



Cell[CellGroupData[{

Cell[47026, 1456, 511, 16, 66, "Input"],

Cell[47540, 1474, 389, 12, 66, "Output"]

}, Open  ]],

Cell[47944, 1489, 104, 3, 32, "Text"],



Cell[CellGroupData[{

Cell[48073, 1496, 445, 13, 66, "Input"],

Cell[48521, 1511, 83, 2, 65, "Output"]

}, Open  ]],

Cell[48619, 1516, 145, 3, 54, "Text"]

}, Open  ]],



Cell[CellGroupData[{

Cell[48801, 1524, 49, 0, 56, "Section"],

Cell[48853, 1526, 175, 4, 54, "Text"],

Cell[49031, 1532, 220, 7, 59, "Input"],

Cell[49254, 1541, 306, 5, 76, "Text"],



Cell[CellGroupData[{

Cell[49585, 1550, 1094, 30, 89, "Input"],

Cell[50682, 1582, 157, 3, 52, "Output"]

}, Open  ]],



Cell[CellGroupData[{

Cell[50876, 1590, 334, 9, 59, "Input"],

Cell[51213, 1601, 148, 3, 52, "Output"]

}, Open  ]],

Cell[51376, 1607, 151, 3, 54, "Text"]

}, Open  ]],



Cell[CellGroupData[{

Cell[51564, 1615, 32, 0, 56, "Section"],

Cell[51599, 1617, 604, 9, 120, "Text"]

}, Open  ]],



Cell[CellGroupData[{

Cell[52240, 1631, 68, 0, 56, "Section"],

Cell[52311, 1633, 518, 8, 120, "Text"],

Cell[52832, 1643, 134, 3, 36, "Input"],

Cell[52969, 1648, 285, 6, 98, "Text"],

Cell[53257, 1656, 218, 5, 76, "Text"],

Cell[53478, 1663, 390, 7, 140, "Input",

  InitializationCell->True],

Cell[53871, 1672, 248, 5, 76, "Text"]

}, Open  ]],



Cell[CellGroupData[{

Cell[54156, 1682, 54, 0, 56, "Section"],

Cell[54213, 1684, 169, 3, 54, "Text"],

Cell[54385, 1689, 90, 1, 36, "Input"],

Cell[54478, 1692, 227, 4, 54, "Text"],

Cell[54708, 1698, 47, 1, 36, "Input"],

Cell[54758, 1701, 159, 3, 54, "Text"],

Cell[54920, 1706, 67, 2, 36, "Input"],

Cell[54990, 1710, 279, 5, 76, "Text"],

Cell[55272, 1717, 370, 11, 36, "Input"],

Cell[55645, 1730, 214, 4, 54, "Text"]

}, Open  ]],



Cell[CellGroupData[{

Cell[55896, 1739, 107, 3, 56, "Section"],

Cell[56006, 1744, 131, 3, 32, "Text"],

Cell[56140, 1749, 68, 1, 36, "Input"],

Cell[56211, 1752, 951, 13, 208, "Text"],

Cell[57165, 1767, 1229, 21, 712, "Input",

  InitializationCell->True],

Cell[58397, 1790, 250, 4, 54, "Text"],

Cell[58650, 1796, 310, 5, 76, "Text"]

}, Open  ]],



Cell[CellGroupData[{

Cell[58997, 1806, 29, 0, 56, "Section"],

Cell[59029, 1808, 456, 7, 98, "Text"]

}, Open  ]]

}

]

*)









(***********************************************************************

End of Mathematica Notebook file.

***********************************************************************)



